Package: rgl
Version: 0.99.9
Title: 3D Visualization Using OpenGL
Author: Daniel Adler <dadler@uni-goettingen.de>, Duncan Murdoch <murdoch@stats.uwo.ca>, and others (see README)
Maintainer: Duncan Murdoch <murdoch@stats.uwo.ca>
Depends: R (>= 3.2.0)
Suggests: MASS, rmarkdown, deldir, orientlib, lattice, misc3d,
        rstudioapi, magick
Imports: graphics, grDevices, stats, utils, htmlwidgets, htmltools,
        knitr, jsonlite (>= 0.9.20), shiny, magrittr, crosstalk
Description: Provides medium to high level functions for 3D interactive graphics, including
    functions modelled on base graphics (plot3d(), etc.) as well as functions for
    constructing representations of geometric objects (cube3d(), etc.).  Output
    may be on screen using OpenGL, or to various standard 3D file formats including
    WebGL, PLY, OBJ, STL as well as 2D image formats, including PNG, Postscript, SVG, PGF.
License: GPL
URL: https://r-forge.r-project.org/projects/rgl/
SystemRequirements: OpenGL, GLU Library, XQuartz (on OSX), zlib
        (optional), libpng (>=1.2.9, optional), FreeType (optional),
        pandoc (>=1.14, needed for vignettes)
BugReports: https://r-forge.r-project.org/projects/rgl/
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2018-01-12 12:21:16 UTC; murdoch
Repository: CRAN
Date/Publication: 2018-01-13 00:19:00 UTC
Built: R 3.4.1; x86_64-w64-mingw32; 2018-08-03 18:54:46 UTC; windows
Archs: x64
